@@ -10,6 +10,7 @@ from __future__ import division
 import collections
 import contextlib
 import errno
+import glob
 import io
 import os
 import pprint
@@ -40,6 +41,7 @@ from psutil.tests import safe_rmpath
 from psutil.tests import sh
 from psutil.tests import skip_on_not_implemented
 from psutil.tests import TESTFN
+from psutil.tests import ThreadTask
 from psutil.tests import TRAVIS
 from psutil.tests import unittest
 from psutil.tests import which
@@ -50,11 +52,11 @@ SIOCGIFADDR = 0x8915
 SIOCGIFCONF = 0x8912
 SIOCGIFHWADDR = 0x8927
 if LINUX:
-    SECTOR_SIZE = psutil._psplatform.SECTOR_SIZE
+    SECTOR_SIZE = 512
 
 
 # =====================================================================
-# utils
+# --- utils
 # =====================================================================
 
 
@@ -140,7 +142,7 @@ def get_free_version_info():
 
 
 # =====================================================================
-# system virtual memory
+# --- system virtual memory
 # =====================================================================
 
 
@@ -374,7 +376,7 @@ class TestSystemVirtualMemory(unittest.TestCase):
 
 
 # =====================================================================
-# system swap memory
+# --- system swap memory
 # =====================================================================
 
 
@@ -436,7 +438,7 @@ class TestSystemSwapMemory(unittest.TestCase):
 
 
 # =====================================================================
-# system CPU
+# --- system CPU
 # =====================================================================
 
 
@@ -466,8 +468,9 @@ class TestSystemCPU(unittest.TestCase):
     def test_cpu_count_logical_w_sysdev_cpu_online(self):
         with open("/sys/devices/system/cpu/online") as f:
             value = f.read().strip()
-        value = int(value.split('-')[1]) + 1
-        self.assertEqual(psutil.cpu_count(), value)
+        if "-" in str(value):
+            value = int(value.split('-')[1]) + 1
+            self.assertEqual(psutil.cpu_count(), value)
 
     @unittest.skipUnless(os.path.exists("/sys/devices/system/cpu"),
                          "/sys/devices/system/cpu does not exist")
@@ -534,9 +537,31 @@ class TestSystemCPU(unittest.TestCase):
             self.assertIsNone(psutil._pslinux.cpu_count_physical())
             assert m.called
 
+    def test_cpu_freq_no_result(self):
+        with mock.patch("psutil._pslinux.glob.glob", return_value=[]):
+            self.assertIsNone(psutil.cpu_freq())
+
+    @unittest.skipIf(TRAVIS, "fails on Travis")
+    def test_cpu_freq_use_second_file(self):
+        # https://github.com/giampaolo/psutil/issues/981
+        def glob_mock(pattern):
+            if pattern.startswith("/sys/devices/system/cpu/cpufreq/policy"):
+                flags.append(None)
+                return []
+            else:
+                flags.append(None)
+                return orig_glob(pattern)
+
+        flags = []
+        orig_glob = glob.glob
+        with mock.patch("psutil._pslinux.glob.glob", side_effect=glob_mock,
+                        create=True):
+            assert psutil.cpu_freq()
+            self.assertEqual(len(flags), 2)
+
 
 # =====================================================================
-# system CPU stats
+# --- system CPU stats
 # =====================================================================
 
 
@@ -557,7 +582,7 @@ class TestSystemCPUStats(unittest.TestCase):
 
 
 # =====================================================================
-# system network
+# --- system network
 # =====================================================================
 
 
@@ -580,7 +605,8 @@ class TestSystemNetwork(unittest.TestCase):
             except RuntimeError:
                 pass
             else:
-                self.assertEqual(stats.isup, 'RUNNING' in out, msg=out)
+                # Not always reliable.
+                # self.assertEqual(stats.isup, 'RUNNING' in out, msg=out)
                 self.assertEqual(stats.mtu,
                                  int(re.findall('MTU:(\d+)', out)[0]))
 
@@ -668,7 +694,7 @@ class TestSystemNetwork(unittest.TestCase):
 
 
 # =====================================================================
-# system disk
+# --- system disk
 # =====================================================================
 
 
@@ -826,7 +852,7 @@ class TestSystemDisks(unittest.TestCase):
 
 
 # =====================================================================
-# misc
+# --- misc
 # =====================================================================
 
 
@@ -985,7 +1011,7 @@ class TestMisc(unittest.TestCase):
         def open_mock(name, *args, **kwargs):
             if PY3 and isinstance(name, bytes):
                 name = name.decode()
-            if name.startswith("/sys/block/sda/queue/hw_sector_size"):
+            if "hw_sector_size" in name:
                 flag.append(None)
                 raise IOError(errno.ENOENT, '')
             else:
@@ -994,19 +1020,185 @@ class TestMisc(unittest.TestCase):
         flag = []
         orig_open = open
         patch_point = 'builtins.open' if PY3 else '__builtin__.open'
+        with mock.patch(patch_point, side_effect=open_mock):
+            psutil.disk_io_counters()
+            self.assertTrue(flag)
+
+    def test_issue_687(self):
+        # In case of thread ID:
+        # - pid_exists() is supposed to return False
+        # - Process(tid) is supposed to work
+        # - pids() should not return the TID
+        # See: https://github.com/giampaolo/psutil/issues/687
+        t = ThreadTask()
+        t.start()
         try:
-            with mock.patch(patch_point, side_effect=open_mock):
-                importlib.reload(psutil._pslinux)
-                importlib.reload(psutil)
-                self.assertEqual(flag, [None])
-                self.assertEqual(psutil._pslinux.SECTOR_SIZE, 512)
+            p = psutil.Process()
+            tid = p.threads()[1].id
+            assert not psutil.pid_exists(tid), tid
+            pt = psutil.Process(tid)
+            pt.as_dict()
+            self.assertNotIn(tid, psutil.pids())
         finally:
-            importlib.reload(psutil._pslinux)
-            importlib.reload(psutil)
+            t.stop()
 
 
 # =====================================================================
-# test process
+# --- sensors
+# =====================================================================
+
+
+@unittest.skipUnless(LINUX, "LINUX only")
+@unittest.skipUnless(hasattr(psutil, "sensors_battery") and
+                     psutil.sensors_battery() is not None,
+                     "no battery")
+class TestSensorsBattery(unittest.TestCase):
+
+    @unittest.skipUnless(which("acpi"), "acpi utility not available")
+    def test_percent(self):
+        out = sh("acpi -b")
+        acpi_value = int(out.split(",")[1].strip().replace('%', ''))
+        psutil_value = psutil.sensors_battery().percent
+        self.assertAlmostEqual(acpi_value, psutil_value, delta=1)
+
+    @unittest.skipUnless(which("acpi"), "acpi utility not available")
+    def test_power_plugged(self):
+        out = sh("acpi -b")
+        if 'unknown' in out.lower():
+            return unittest.skip("acpi output not reliable")
+        plugged = "Charging" in out.split('\n')[0]
+        self.assertEqual(psutil.sensors_battery().power_plugged, plugged)
+
+    def test_emulate_power_plugged(self):
+        # Pretend the AC power cable is connected.
+        def open_mock(name, *args, **kwargs):
+            if name.startswith("/sys/class/power_supply/AC0/online"):
+                return io.BytesIO(b"1")
+            else:
+                return orig_open(name, *args, **kwargs)
+
+        orig_open = open
+        patch_point = 'builtins.open' if PY3 else '__builtin__.open'
+        with mock.patch(patch_point, side_effect=open_mock) as m:
+            self.assertEqual(psutil.sensors_battery().power_plugged, True)
+            self.assertEqual(
+                psutil.sensors_battery().secsleft, psutil.POWER_TIME_UNLIMITED)
+            assert m.called
+
+    def test_emulate_power_not_plugged(self):
+        # Pretend the AC power cable is not connected.
+        def open_mock(name, *args, **kwargs):
+            if name.startswith("/sys/class/power_supply/AC0/online"):
+                return io.BytesIO(b"0")
+            elif name.startswith("/sys/class/power_supply/BAT0/status"):
+                return io.BytesIO(b"discharging")
+            else:
+                return orig_open(name, *args, **kwargs)
+
+        orig_open = open
+        patch_point = 'builtins.open' if PY3 else '__builtin__.open'
+        with mock.patch(patch_point, side_effect=open_mock) as m:
+            self.assertEqual(psutil.sensors_battery().power_plugged, False)
+            assert m.called
+
+    def test_emulate_power_undetermined(self):
+        # Pretend we can't know whether the AC power cable not
+        # connected (assert fallback to False).
+        def open_mock(name, *args, **kwargs):
+            if name.startswith("/sys/class/power_supply/AC0/online") or \
+                    name.startswith("/sys/class/power_supply/AC/online"):
+                raise IOError(errno.ENOENT, "")
+            elif name.startswith("/sys/class/power_supply/BAT0/status"):
+                return io.BytesIO(b"???")
+            else:
+                return orig_open(name, *args, **kwargs)
+
+        orig_open = open
+        patch_point = 'builtins.open' if PY3 else '__builtin__.open'
+        with mock.patch(patch_point, side_effect=open_mock) as m:
+            self.assertIsNone(psutil.sensors_battery().power_plugged)
+            assert m.called
+
+    def test_emulate_no_base_files(self):
+        # Emulate a case where base metrics files are not present,
+        # in which case we're supposed to get None.
+        def open_mock(name, *args, **kwargs):
+            if name.startswith("/sys/class/power_supply/BAT0/energy_now") or \
+                    name.startswith("/sys/class/power_supply/BAT0/charge_now"):
+                raise IOError(errno.ENOENT, "")
+            else:
+                return orig_open(name, *args, **kwargs)
+
+        orig_open = open
+        patch_point = 'builtins.open' if PY3 else '__builtin__.open'
+        with mock.patch(patch_point, side_effect=open_mock) as m:
+            self.assertIsNone(psutil.sensors_battery())
+            assert m.called
+
+    def test_emulate_energy_full_0(self):
+        # Emulate a case where energy_full files returns 0.
+        def open_mock(name, *args, **kwargs):
+            if name.startswith("/sys/class/power_supply/BAT0/energy_full"):
+                return io.BytesIO(b"0")
+            else:
+                return orig_open(name, *args, **kwargs)
+
+        orig_open = open
+        patch_point = 'builtins.open' if PY3 else '__builtin__.open'
+        with mock.patch(patch_point, side_effect=open_mock) as m:
+            self.assertEqual(psutil.sensors_battery().percent, 0)
+            assert m.called
+
+    def test_emulate_energy_full_not_avail(self):
+        # Emulate a case where energy_full file does not exist.
+        # Expected fallback on /capacity.
+        def open_mock(name, *args, **kwargs):
+            if name.startswith("/sys/class/power_supply/BAT0/energy_full"):
+                raise IOError(errno.ENOENT, "")
+            elif name.startswith("/sys/class/power_supply/BAT0/capacity"):
+                return io.BytesIO(b"88")
+            else:
+                return orig_open(name, *args, **kwargs)
+
+        orig_open = open
+        patch_point = 'builtins.open' if PY3 else '__builtin__.open'
+        with mock.patch(patch_point, side_effect=open_mock) as m:
+            self.assertEqual(psutil.sensors_battery().percent, 88)
+            assert m.called
+
+    def test_emulate_no_ac0_online(self):
+        # Emulate a case where /AC0/online file does not exist.
+        def path_exists_mock(name):
+            if name.startswith("/sys/class/power_supply/AC0/online"):
+                return False
+            else:
+                return orig_path_exists(name)
+
+        orig_path_exists = os.path.exists
+        with mock.patch("psutil._pslinux.os.path.exists",
+                        side_effect=path_exists_mock) as m:
+            psutil.sensors_battery()
+            assert m.called
+
+    def test_emulate_no_power(self):
+        # Emulate a case where /AC0/online file nor /BAT0/status exist.
+        def open_mock(name, *args, **kwargs):
+            if name.startswith("/sys/class/power_supply/AC/online") or \
+                    name.startswith("/sys/class/power_supply/AC0/online") or \
+                    name.startswith("/sys/class/power_supply/BAT0/status"):
+                raise IOError(errno.ENOENT, "")
+            else:
+                return orig_open(name, *args, **kwargs)
+
+        orig_open = open
+        patch_point = 'builtins.open' if PY3 else '__builtin__.open'
+        with mock.patch(patch_point, side_effect=open_mock) as m:
+            self.assertIsNone(psutil.sensors_battery().power_plugged)
+            assert m.called
+
+
+# =====================================================================
+# --- test process
 # =====================================================================
 
 
@@ -1018,49 +1210,6 @@ class TestProcess(unittest.TestCase):
 
     tearDown = setUp
 
-    def test_compare_stat_and_status_files(self):
-        # /proc/pid/stat and /proc/pid/status have many values in common.
-        # Whenever possible, psutil uses /proc/pid/stat (it's faster).
-        # For all those cases we check that the value found in
-        # /proc/pid/stat (by psutil) matches the one found in
-        # /proc/pid/status.
-        p = psutil.Process()
-        with psutil._psplatform.open_text('/proc/%s/status' % p.pid) as f:
-            for line in f:
-                line = line.strip()
-                if line.startswith('Name:'):
-                    name = line.split()[1]
-                    # Name is truncated to 15 chars
-                    self.assertEqual(p.name()[:15], name[:15])
-                elif line.startswith('State:'):
-                    status = line[line.find('(') + 1:line.rfind(')')]
-                    status = status.replace(' ', '-')
-                    self.assertEqual(p.status(), status)
-                elif line.startswith('PPid:'):
-                    ppid = int(line.split()[1])
-                    self.assertEqual(p.ppid(), ppid)
-                # The ones below internally are determined by reading
-                # 'status' file but we use a re to extract the info
-                # so it makes sense to check them.
-                elif line.startswith('Threads:'):
-                    num_threads = int(line.split()[1])
-                    self.assertEqual(p.num_threads(), num_threads)
-                elif line.startswith('Uid:'):
-                    uids = tuple(map(int, line.split()[1:4]))
-                    self.assertEqual(tuple(p.uids()), uids)
-                elif line.startswith('Gid:'):
-                    gids = tuple(map(int, line.split()[1:4]))
-                    self.assertEqual(tuple(p.gids()), gids)
-                elif line.startswith('voluntary_ctxt_switches:'):
-                    vol = int(line.split()[1])
-                    self.assertAlmostEqual(
-                        p.num_ctx_switches().voluntary, vol, delta=2)
-                elif line.startswith('nonvoluntary_ctxt_switches:'):
-                    invol = int(line.split()[1])
-                    self.assertAlmostEqual(
-                        p.num_ctx_switches().involuntary, invol,
-                        delta=2)
-
     def test_memory_full_info(self):
         src = textwrap.dedent("""
             import time
@@ -1169,13 +1318,6 @@ class TestProcess(unittest.TestCase):
             p.cmdline() == ['foo', 'bar', '']
             assert m.called
 
-    def test_io_counters_mocked(self):
-        with mock.patch('psutil._pslinux.open', create=True) as m:
-            self.assertRaises(
-                NotImplementedError,
-                psutil._pslinux.Process(os.getpid()).io_counters)
-            assert m.called
-
     def test_readlink_path_deleted_mocked(self):
         with mock.patch('psutil._pslinux.os.readlink',
                         return_value='/home/foo (deleted)'):
@@ -1232,5 +1374,84 @@ class TestProcess(unittest.TestCase):
                 self.assertRaises(psutil.ZombieProcess, psutil.Process().exe)
 
 
+@unittest.skipUnless(LINUX, "LINUX only")
+class TestProcessAgainstStatus(unittest.TestCase):
+    """/proc/pid/stat and /proc/pid/status have many values in common.
+    Whenever possible, psutil uses /proc/pid/stat (it's faster).
+    For all those cases we check that the value found in
+    /proc/pid/stat (by psutil) matches the one found in
+    /proc/pid/status.
+    """
+
+    @classmethod
+    def setUpClass(cls):
+        cls.proc = psutil.Process()
+
+    def read_status_file(self, linestart):
+        with psutil._psplatform.open_text(
+                '/proc/%s/status' % self.proc.pid) as f:
+            for line in f:
+                line = line.strip()
+                if line.startswith(linestart):
+                    value = line.partition('\t')[2]
+                    try:
+                        return int(value)
+                    except ValueError:
+                        return value
+            else:
+                raise ValueError("can't find %r" % linestart)
+
+    def test_name(self):
+        value = self.read_status_file("Name:")
+        self.assertEqual(self.proc.name(), value)
+
+    def test_status(self):
+        value = self.read_status_file("State:")
+        value = value[value.find('(') + 1:value.rfind(')')]
+        value = value.replace(' ', '-')
+        self.assertEqual(self.proc.status(), value)
+
+    def test_ppid(self):
+        value = self.read_status_file("PPid:")
+        self.assertEqual(self.proc.ppid(), value)
+
+    def test_num_threads(self):
+        value = self.read_status_file("Threads:")
+        self.assertEqual(self.proc.num_threads(), value)
+
+    def test_uids(self):
+        value = self.read_status_file("Uid:")
+        value = tuple(map(int, value.split()[1:4]))
+        self.assertEqual(self.proc.uids(), value)
+
+    def test_gids(self):
+        value = self.read_status_file("Gid:")
+        value = tuple(map(int, value.split()[1:4]))
+        self.assertEqual(self.proc.gids(), value)
+
+    @retry_before_failing()
+    def test_num_ctx_switches(self):
+        value = self.read_status_file("voluntary_ctxt_switches:")
+        self.assertEqual(self.proc.num_ctx_switches().voluntary, value)
+        value = self.read_status_file("nonvoluntary_ctxt_switches:")
+        self.assertEqual(self.proc.num_ctx_switches().involuntary, value)
+
+    def test_cpu_affinity(self):
+        value = self.read_status_file("Cpus_allowed_list:")
+        if '-' in str(value):
+            min_, max_ = map(int, value.split('-'))
+            self.assertEqual(
+                self.proc.cpu_affinity(), list(range(min_, max_ + 1)))
+
+    def test_cpu_affinity_eligible_cpus(self):
+        value = self.read_status_file("Cpus_allowed_list:")
+        with mock.patch("psutil._pslinux.per_cpu_times") as m:
+            self.proc._proc._get_eligible_cpus()
+        if '-' in str(value):
+            assert not m.called
+        else:
+            assert m.called
+
+
 if __name__ == '__main__':
     run_test_module_by_name(__file__)
