@@ -388,7 +388,7 @@ class TestScripts(unittest.TestCase):
             src = f.read()
         ast.parse(src)
 
-    def test_check_presence(self):
+    def test_coverage(self):
         # make sure all example scripts have a test method defined
         meths = dir(self)
         for name in os.listdir(SCRIPTS_DIR):
@@ -466,6 +466,35 @@ class TestScripts(unittest.TestCase):
     def test_winservices(self):
         self.assert_stdout('winservices.py')
 
+    def test_cpu_distribution(self):
+        self.assert_syntax('cpu_distribution.py')
+
+    @unittest.skipIf(TRAVIS, "unreliable on travis")
+    def test_temperatures(self):
+        if hasattr(psutil, "sensors_temperatures") and \
+                psutil.sensors_temperatures():
+            self.assert_stdout('temperatures.py')
+        else:
+            self.assert_syntax('temperatures.py')
+
+    @unittest.skipIf(TRAVIS, "unreliable on travis")
+    def test_fans(self):
+        if hasattr(psutil, "sensors_fans") and psutil.sensors_fans():
+            self.assert_stdout('fans.py')
+        else:
+            self.assert_syntax('fans.py')
+
+    def test_battery(self):
+        if hasattr(psutil, "sensors_battery") and \
+                psutil.sensors_battery() is not None:
+            self.assert_stdout('battery.py')
+        else:
+            self.assert_syntax('battery.py')
+
+    @unittest.skipIf(APPVEYOR or TRAVIS, "unreliable on CI")
+    def test_sensors(self):
+        self.assert_stdout('sensors.py')
+
 
 # ===================================================================
 # --- Unit tests for test utilities.
